using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Assignment2
{
    partial class MainProgram
    {
        public delegate void Handler(string[] arguments);
        class CommandHandler
        {
            public readonly uint maxArgs;      // The maximum number of arguments accepted by the command
            public readonly string syntaxHelp; // Message for users entering too many arguments
            public readonly Handler h;         // The actual method performing the command
            public CommandHandler(string command, uint m, string s, Handler h)
            {
                maxArgs = m;
                syntaxHelp = s;
                this.h = h;
                if (CommandRepository == null)
                {
                    CommandRepository = new Dictionary<string, CommandHandler>();
                }
                CommandRepository[command] = this;
            }
        }
        // Holds the Command handlers for all commands recognized by the application
        private static Dictionary<string, CommandHandler> CommandRepository;

        static void Main(string[] args)
        {
            Console.WriteLine("Welcome to the console interface to the student information system!");
            bool done = false;
            Regex separator = new Regex(@"[ \t]+");
            while (!done)
            {
                Console.Write("Enter commnad>");
                string fullCommand = Console.ReadLine();
                string[] comandAndArgs = separator.Split(fullCommand.Trim());
                string command = comandAndArgs[0].ToLower();
                if ( command == "exit" || command == "quit")
                {
                    done = true;
                }
                else if (CommandRepository.ContainsKey(command))
                {
                    CommandHandler ch = CommandRepository[command];
                    if (comandAndArgs.Length - 1 > ch.maxArgs)
                    {
                        Console.WriteLine("  Invalid syntax for command {0} correct syntax is:", command);
                        Console.WriteLine("  "+ch.syntaxHelp);
                    }
                    else
                    {
                        try
                        {
                            ch.h(comandAndArgs);
                        }
                        catch (Exception ex)
                        {
                            Console.WriteLine("  Caught exception " + ex.Message);
                        }
                    }
                }
                else
                {
                    Console.WriteLine("  Command \"{0}\" not valid",command);
                }
            }
        }
    }
}
